Package: covRNA
Type: Package
Title: Multivariate Analysis of Transcriptomic Data
Version: 1.38.0
Author: Lara Urban <lara.h.urban@ebi.ac.uk>
Maintainer: Lara Urban <lara.h.urban@ebi.ac.uk>
Description: This package provides the analysis methods fourthcorner and RLQ
    analysis for large-scale transcriptomic data.
License: GPL (>= 2)
LazyData: TRUE
Depends: ade4, Biobase
Imports: parallel, genefilter, grDevices, stats, graphics
biocViews: GeneExpression, Transcription
Suggests: BiocStyle, knitr, rmarkdown
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/covRNA
git_branch: RELEASE_3_23
git_last_commit: 84cc5ca
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:48:16 UTC; biocbuild
Built: R 4.6.0; ; 2026-04-29 04:38:32 UTC; unix

# import
import(parallel)
import(ade4)
import(Biobase)
import(genefilter)

import(grDevices)
import(stats)
importFrom("graphics","barplot","mtext","par","plot.default","rect","segments","strwidth","text")

# export
export(stat,ord,vis,plot.stat,plot.ord)

# S3methods
S3method(plot,stat)
S3method(plot,ord)

# covRNA

The R package covRNA provides a convenient and fast interface for testing and visualizing the relationship between sample and gene covariates mediated by gene expression data. The fourthcorner analysis tests the statistical significance of sample and gene covariate relationships by permutation tests while the RLQ visualizes relationships within and between sample and gene covariates.

The method is based on the powerful fourthcorner and RLQ analyses used in ecological research for the analysis of species abundance data. We have modified the algorithms of the R package ade4 to make the method suitable to the distributional characteristics of RNA-Seq read counts and microarray intensities and to provide a parallelized high-performance implementation to allow for the analysis of large-scale transcriptomics data. We further supply the user with methods for unsupervised gene filtering and plotting functions to ease the analysis workflow. cov-RNA can be easily applied to microarray or RNA-Seq data of any organism. We provide a vignette with an exemplified analysis of an RNA-Seq dataset of Bacillus anthracis.

To cite covRNA, please use citation("covRNA"). For further details, please refer to the vignette and the man pages.

# visualisation of different features of x

# input:
# [x] object of x function;
# [feature] defines which features shall be visualised:
# "columns L","rows L", "columns R", "columns Q", "variance",
# "correlation circle R" or "correlation circle Q" (default: "variance");
# [xaxis] and [yaxis] define axes of xination (default:1 and 2, resp.);
# [cex] defines text size (default: 1);
# [rangex, rangey] define ize of the plot (default=2);
# [...] additional features can be added

plot.ord <- function(x, feature="variance", xaxis=1, yaxis=2,
                         cex=1, range=2, ...) {

  if (feature=="rows L") {
    plot(range * x$lR[, c(xaxis, yaxis)], type='n', xlab=paste("axis", xaxis),
        ylab=paste("axis", yaxis), bty='l')
    text(x$lR[,1], x$lR[,2], row.names(x$lR), cex=cex)
  }
    
  if (feature=="columns L") {
    plot(range * x$lQ[, c(xaxis, yaxis)], type='n', xlab=paste("axis", xaxis),
        ylab=paste("axis", yaxis), bty='l')
    text(x$lQ[,1], x$lQ[,2], row.names(x$lQ), cex=cex)
  }
    
  if (feature=="correlation circle R") {
    plot(range * rbind(x$aR,c(0,0)), type='n', bty='l')
    text(x$aR[,1], x$aR[,2], row.names(x$aR), cex=1)
    text(0,0, "0")
    segments(0,0,x$aR[1,1], x$aR[1,2])
    segments(0,0,x$aR[2,1], x$aR[2,2])
  }
    
  if (feature=="correlation circle Q") {
    plot(range * rbind(x$aQ,c(0,0)), type='n', bty='l')
    text(x$aQ[,1], x$aQ[,2], row.names(x$aQ), cex=1)
    text(0,0, "0")
    segments(0,0,x$aQ[1,1], x$aQ[1,2])
    segments(0,0,x$aQ[2,1], x$aQ[2,2])    
  }
    
  if (feature=="columns R") {
    plot(range * x$l1[, c(xaxis, yaxis)], type='n', xlab=paste("axis", xaxis),
         ylab=paste("axis", yaxis), bty='l')
    text(x$l1[,1], x$l1[,2], row.names(x$l1), cex=cex)
  }
    
  if (feature=="columns Q") {
    plot(range * x$c1[, c(xaxis, yaxis)], type='n', xlab=paste("axis", xaxis),
        ylab=paste("axis", yaxis), bty='l')
    text(x$c1[,1], x$c1[,2], row.names(x$c1), cex=cex)
  }
    
  if (feature=="variance") {
    barplot(x$variance, col=c(rep("black", x$nf),
                              rep("grey", length(x$variance) - x$nf)),
            names=1:length(x$variance),
            ylab="amount of explained variance",
            ylim=c(0, max(max(x$variance) + 0.1,1)))
  }
}

# rlq analysis of covaRNA: ord

# input:
# [ExprSet] ExpressionSet: assayData will be called L (non-negative
# values), phenoData Q and featureData R (default);
# or: [R, L, Q] single dataframes;
# [exprvar] for gene filtering: may vary between 0 and 1 (=default)
# & indicates fraction of most variably expressed genes to be analysed;
# [nf] number of axes to be considered by ordination

# output: list of class ord:
# all features of an rlq object (see ade4)
# [ngenes] number of analyzed genes;
# [call] shows called function
# [variance] variance explained by the considered axes in [%]

ord <- function(ExprSet, R=NULL, L=NULL, Q=NULL, exprvar=1, nf=2) {

  # examine and transform data
    # if ExpressionSet is missing: use single dataframes as input
    if (missing(ExprSet)) {
        
        if (missing(R)) {
            R = as.data.frame(diag(x=1, nrow=nrow(L), ncol=nrow(L)))
            warning("R is missing and is replaced by an identity matrix.")
        }
        
        if (missing(Q)) {
            Q = as.data.frame(diag(x=1, nrow=ncol(L), ncol=ncol(L)))
            warning("R is missing and is replaced by an identity matrix.")
        }
        
        if (missing(L)) {
            stop("L is missing.")
        }
        
        if (is.matrix(L)) matL <- L else 
            if (is.data.frame(L)) matL <- as.matrix(L) else 
                stop("L has to be a dataframe or a matrix")
        
        if (is.matrix(R)) {
            tabR <- as.data.frame(R)
            warning("R is transformed to a dataframe by as.data.frame().")
        } else if (is.data.frame(R)) tabR <- R else 
            stop("R has to be a dataframe or a matrix")
        
        if (is.matrix(Q)) {
            tabQ <- as.data.frame(Q)
            warning("Q is transformed to a dataframe by as.data.frame().")
        } else if (is.data.frame(Q)) tabQ <- Q else 
            stop("Q has to be a dataframe or a matrix")
    } else {
        # else: transform ExpressionSet to matrix/dataframes
        matL <- exprs(ExprSet)
        tabR <- fData(ExprSet)
        tabQ <- pData(ExprSet)
    }
  
  # control values of the dataframes
  if (any(is.na(tabR)))
    stop("NA in R")
  if (any(is.na(tabQ)))
    stop("NA in Q")
  if (any(is.na(matL)))
    stop("NA in L")
  if (any(matL<0))
    stop("negative values in L")

  # control features of the dataframes
  if (nrow(tabR) != nrow(matL))
    stop("row number of R does not suit row number of L")
  if (nrow(tabQ) != ncol(matL))
    stop("row number of Q does not suit column number of L")

  # filter gene by gene expression variance according to exprvar
  if (exprvar != 1) {
    # determine number of genes by given fraction
    ngenes <- trunc(exprvar * nrow(matL))
    # filter L to retain the most variably expressed genes
    matL <- matL[order(rowVars(matL), decreasing=TRUE)[1:ngenes],]
    # just use gene covariates in R which contain at least one annotation after
    # the unsupervised filtering
    whichR <- unique(which(colSums(tabR)==0))
    tabR <- tabR[,-whichR]
  }

  # get variable type of R and Q
  varR <- vector(mode="logical", length=dim(tabR)[2])
  for (i in 1:dim(tabR)[2]) {
      varR[i] <- is.numeric(tabR[,i])
  }
  
  varQ <- vector(mode="logical", length=dim(tabQ)[2])
  for (i in 1:dim(tabQ)[2]) {
      varQ[i] <- is.numeric(tabQ[,i])
  }

  # visualize the three tables with respect to variable type of R/Q

  # matL is visualised by CA
  VL <- dudi.coa(matL, scannf=FALSE)

  # tabR and tabQ are visulaised by PCA, Hillsmith or MCA
  if (all(varR)) VR <- dudi.pca(tabR, row.w=VL$lw, scannf=FALSE) else 
  if (any(varR)) VR <- dudi.hillsmith(tabR, row.w=VL$lw, scannf=FALSE) else 
  VR <- dudi.acm(tabR, row.w=VL$lw, scannf=FALSE)

  if (all(varQ)) VQ <- dudi.pca(tabQ, row.w=VL$cw, scannf=FALSE) else 
  if (any(varQ)) VQ <- dudi.hillsmith(tabQ, row.w=VL$cw, scannf=FALSE) else 
  VQ <-dudi.acm(tabQ, row.w=VL$cw, scannf=FALSE)

  # RLQ is based on the three singular matrix ordinations
  result <- rlq(VR, VL, VQ, scannf=FALSE, nf=2)

  # amount of variance explained by the axes
  result$variance <- (100*result$eig/sum(result$eig))
  # add number of analysed genes to result
  if (exprvar != 1) {result$ngenes <- ngenes} else result$ngenes <- "all"
  # add call of the function to result
  result$call <- match.call()
  class(result) <- "ord"
  return(result)
}

# visualisation of the stat results as table of covariate combinations

# input:
# [x] object of stat function;
# [col] three colours: first colour defines non-significant associations,
# second/third colour define significant positive/negative asscoiations
# (default: "lightgrey","deepskyblue","red");
# [sig] if TRUE, only covariates involved in at least one significant
# association are shown (default: TRUE)
# [alpha] defines significance level of pvalues (default:0.05);
# [show] defines which p-values shall be chosen (default:adj);
# [cex] defines text size of table;
# [ynames] and [xnames] define row and column names of the table (optional);
# [ytext] and [xtext] define how text shall be rotated (default: 1 for
# horizontal; 3 for vertical);
# [shiftx] and [shifty] define how much the text should be shifted to the
# right/top (if>0) or to the left/bottom (if<0);
# [...] additional features can be added

plot.stat <- function(x, col=c("lightgrey","deepskyblue","red"), sig=TRUE,
                      alpha=0.05, show=c("adj","non-adj"), cex=1,
                      ynames, xnames, ytext=1, xtext=1, shiftx=0, shifty=0,
                      ...) {
    
  # save statistical values as matrix
  stat <- x$stat
  statnum <- as.numeric(stat)

  # save either adjusted or non-adjusted p-values as matrix (default:adj)
  show <- match.arg(show)
  if (show=="non-adj") {
    statpvalm <- x$pvalue
  } else {
    statpvalm <- x$adj.pvalue 
  }
  
  if (sig == TRUE) {
    rows <- unique(which(statpvalm<=alpha, arr.ind = TRUE)[,1])
    columns <- unique(which(statpvalm<=alpha, arr.ind = TRUE)[,2])
    statpvalm <- statpvalm[rows, columns]
    statpval <- as.numeric(statpvalm)
    stat <- stat[rows, columns]
    statnum <- as.numeric(stat)
  }

  # choose significant associations
  statsel <- which(statpval <= alpha)

  # init vector des with 1; if des==1: no significant association
  des <- rep(1, length(statnum))

  # set des=2 for postive significant associations and des=3 for
  # negative significant associations
  des[statsel] <- ifelse(statnum[statsel] > 0,2,3)

  # define parameters for plotting a table
  xx <- 1:ncol(stat)
  yy <- nrow(stat):1
  wx <- range(xx)
  wy <- range(yy)
  dx <- diff(wx)/(length(xx)-1)/2
  dy <- diff(wy)/(length(yy)-1)/2
  xlim <- wx + c(-dx, dx)
  ylim <- wy +c (-dy, dy)

  # set graphical parameters
  strx <- max(strwidth(paste(" ", rownames(stat), " ", sep =""),
                         units="inches", cex=cex))
  stry <- max(strwidth(paste(" ", colnames(stat), " ", sep =""),
                         units = "inches", cex=cex))
  par(mai=c(0.2, strx, stry, 0.1))

  # plot an empty table
  plot.default(0, 0, type="n", xlab="", ylab="",
               xaxt="n", yaxt="n", xlim=xlim, ylim=ylim,
               xaxs="i", yaxs="i", frame.plot=FALSE)

  # add text to the rows and columns of the table
  # if no names are given as input, take row- and columnnames of stat
  if (missing(ynames)) ynames <- rownames(stat)
  if (missing(xnames)) xnames <- colnames(stat)
  # plot names next to table
  ynew <- seq(wy[1], wy[2], le=length(yy))[rank(yy)]
  mtext(at=ynew+shifty, side=2, text=paste(ynames," ",sep=""), adj=1,
        cex=cex, las=ytext)
  xnew <- seq(wx[1], wx[2], le=length(xx))[rank(xx)]
  mtext(at=xnew+shiftx, side=3, text=paste(xnames," ",sep=""), adj=0,
        cex=cex, las=xtext)

  # add coloured rectangles to the table
  xtot <- xx[col(stat)]
  ytot <- yy[row(stat)]
  xdelta <- 0.9*(max(xx)-min(xx))/(length(xx)-1)/2
  ydelta <- 0.9*(max(yy)-min(yy))/(length(yy)-1)/2
  rect(xtot-xdelta, ytot-ydelta, xtot+xdelta, ytot+ydelta, col=col[1:3][des],
       border="black")
}

# statistical analysis of covRNA: stat

# input:
# [ExprSet] ExpressionSet: assayData will be called L (non-negative
# values), phenoData Q and featureData R (default);
# or: [R, L, Q] single dataframes;
# [npermut] number of permutations;
# [padjust] multiple hypothesis testing adjustment method;
# [nrcor] number of cores that shall be used (default:all available-1);
# [exprvar] for gene filtering: may vary between 0 and 1 (=default)
# & indicates fraction of most variably expressed genes to be analysed;

# output: list of class stat:
# [pvalue], [adj.pvalue], [stat] matrices of adjusted, non-adjusted
# p-values and statistical values;
# [ngenes] number of analyzed genes;
# [adjust.method] multiple hypothesis testing adjustment method
# (default: Benjamini and Hochberg)
# [npermut] number of permutations (default: 9999)
# [call] shows called function

# statistics:
# quant-quant: correlation coefficient of normalised variables
# qual-qual: Chi-square contingency table of binary variables
# qual-quant: correlation coefficient

stat <- function(ExprSet, R=NULL, L=NULL, Q=NULL, npermut=9999, padjust="BH", nrcor=detectCores()-1, exprvar=1) {

  #####

  # subfunctions:

  # 1. "disj" returns the disjunctive table of a factor vector
  disj <- function(fac) {
    # initialise matrix with levels of the factor as columns
    mat <- matrix(0, nrow = length(fac), ncol = nlevels(fac))
    # set respective values to 1, the rest remains 0
    mat[(1:length(fac)) + length(fac) * (unclass(fac)-1)] <- 1
    # give row- and columnames to the matrix
    dimnames(mat) <- list(names(fac), as.character(levels(fac)))
    # return the matrix as dataframe
    return(data.frame(mat, check.names = FALSE))
  }

  # 2. "normalise" normalises R/Q by the row/column weights of L
  normalise <- function (mtx, weight, sumL) {
    # mtx as matrix R or Q to normalise, weight as row/column weight of
    # matrix L and sumL as sum of entire matrix L
    # row-wise mean of mtx
    mn <- 0
    mn <- sum(mtx * (weight/sumL))
    # row-wise variance of mtx
    vr <- 0
    vr <- sum((weight/sumL) * (mtx-mn)^2)
    # set variances <= 0 to 1 to avoid effect
    if (vr<=0) vr <- 1
    # calculate standard deviation
    vr <- sqrt(vr)
    # normalise each column of mtx
    mtx <- (mtx-mn)/vr
    return(mtx)
  }

  # 3. "calstat" permutes matrix L row- or columnwise once and counts how
  # often empirical statistical values >= observed statistical value
  calstat <- function(R, Rq, L, Q, Qq, o, permut, ind) {
    # permut="r" indicates row-wise permutation
    # permut="c" indicates column-wise permutation
    if (permut=="r") {Lpermut <- L[sample(1:nrow(L)),]}
    if (permut=="c") {Lpermut <- L[,sample(1:ncol(L))]}
    # calculate permuted statistic of quantitative and categorical
    newobsc <- t(R) %*% (Lpermut %*% Q)
    newobsq <- t(Rq) %*% (Lpermut %*% Qq)
    # dependent on index, decide which observed value shall be chosen
    newobs <- matrix(sapply(1:prod(dim(ind)), function(x)
                        if (as.numeric(ind)[x]==4) as.numeric(newobsc)[x]
                        else as.numeric(newobsq)[x]),
                     nrow=nrow(ind),
                     ncol=ncol(ind))
    # count how often permuted statistic is larger than observed one
    return(newobs>=o)
  }


  #####

  # examine and transform data

  # if ExpressionSet is missing: use single dataframes as input
    if (missing(ExprSet)) {
        if (missing(R)) {
            R = as.data.frame(diag(x=1, nrow=nrow(L), ncol=nrow(L)))
            warning("R is missing and is replaced by an identity matrix.")
        }
        if (missing(Q)) {
            Q = as.data.frame(diag(x=1, nrow=ncol(L), ncol=ncol(L)))
            warning("R is missing and is replaced by an identity matrix.")
        }
        if (missing(L)) {
            stop("L is missing.")
        }

        if (is.matrix(L)) matL <- L else
            if (is.data.frame(L)) matL <- as.matrix(L) else
                stop("L has to be a dataframe or a matrix")

            if (is.matrix(R)) {
                tabR <- as.data.frame(R)
                warning("R is transformed to a dataframe by as.data.frame().")
            } else if (is.data.frame(R)) tabR <- R else
                stop("R has to be a dataframe or a matrix")

            if (is.matrix(Q)) {
                tabQ <- as.data.frame(Q)
                warning("Q is transformed to a dataframe by as.data.frame().")
            } else if (is.data.frame(Q)) tabQ <- Q else
                stop("Q has to be a dataframe or a matrix")
    } else {
        # else: transform ExpressionSet to matrix/dataframes
    matL <- exprs(ExprSet)
    tabR <- fData(ExprSet)
    tabQ <- pData(ExprSet)
  }

  # control values of the dataframes
  if (any(is.na(tabR)))
    stop("NA in R")
  if (any(is.na(tabQ)))
    stop("NA in Q")
  if (any(is.na(matL)))
    stop("NA in L")
  if (any(matL<0))
    stop("negative values in L")

  # control features of the dataframes
  if (nrow(tabR) != nrow(matL))
    stop("row number of R does not match row number of L")
  if (nrow(tabQ) != ncol(matL))
    stop("row number of Q does not match column number of L")

  # filter gene by gene expression variance according to exprvar
  if (exprvar != 1) {
    # determine number of genes by given fraction
    ngenes <- trunc(exprvar * nrow(matL))
    # filter L to retain the most variably expressed genes
    matL <- matL[order(rowVars(matL), decreasing=TRUE)[1:ngenes],]
    # just use gene covariates in R which contain at least one annotation after
    # the unsupervised filtering
    whichR <- unique(which(colSums(tabR)==0))
    tabR <- tabR[,-whichR]
  }

  # transform R and Q to matrices and order columns alphabetically
  # R:
  # init matrix
  matR <- matrix(0, nrow(tabR), 1)
  # init colnames of matrix
  namesR <- "init"
  # init indexR which will assign 1 to quantitative and 2 to
  # categorical variables
  indexR <- "init"
  for (i in 1:ncol(tabR)) {
    # if variable is quantitative: add it to new matrix
    if (is.numeric(tabR[,i])) {
      matR <- cbind(matR, tabR[,i])
      namesR <- c(namesR, names(tabR)[i])
      indexR <- c(indexR, 1)
    }
    # if variable is categorical: add disjunctive table to matrix
    else if (is.factor(tabR[,i])) {
      disjtabR <- disj(tabR[,i])
      matR <- cbind(matR, disjtabR)
      # colnames of the variables will contain variable and level name
      namesR <- c(namesR, paste(names(tabR)[i], ".", names(disjtabR)))
      indexR <- c(indexR, rep(2, ncol(disjtabR)))
    }
  }
  # delete the initialisations
  matR <- as.matrix(matR[,-1])
  colnames(matR) <- namesR[-1]
  rm(namesR)
  indexR <- indexR[-1]
  # order the variables alphabetically
  if (dim(matR)[2] !=1) {
      indexR <- indexR[order(colnames(matR))]
      matR <- matR[, order(colnames(matR))]
      }
  
  


  # Q: do the same with matrix Q (see matrix R)
  matQ <- matrix(0, nrow(tabQ), 1)
  namesQ <- "init"
  indexQ <- "init"
  for (i in 1:ncol(tabQ)) {
    if (is.numeric(tabQ[,i])) {
      matQ <- cbind(matQ, tabQ[,i])
      namesQ <- c(namesQ, names(tabQ)[i])
      indexQ <- c(indexQ, 1)
    }
  else if (is.factor(tabQ[,i])) {
    disjtabQ <- disj(tabQ[,i])
    matQ <- cbind(matQ, disjtabQ)
    namesQ <- c(namesQ, paste(names(tabQ)[i], ".", names(disjtabQ)))
    indexQ <- c(indexQ, rep(2, ncol(disjtabQ)))
    }
  }
  matQ <- as.matrix(matQ[,-1])
  colnames(matQ) <- namesQ[-1]
  rm(namesQ)
  indexQ <- indexQ[-1]
  if (dim(matQ)[2] !=1) {
      indexQ <- indexQ[order(colnames(matQ))]
      matQ <- matQ[, order(colnames(matQ))]
      }
  

  # calculate index as sum of indexR and indexQ; only index==4 is interesting,
  # since the both variables are categorical (important for
  # calculation of statistical values)
  index <- matrix(as.numeric(rep(indexR, length(indexQ))),
                  nrow=length(indexR), ncol=length(indexQ))
  index <- matrix(apply(index, 1, function(x) (x + as.numeric(indexQ))),
                    nrow = length(indexR), ncol = length(indexQ), byrow=TRUE)

  # normalise quantitative variables of R and Q per column and save these
  # in matRq and matQq
  matRq <- apply(matR, 2, function(x) normalise(x, rowSums(matL), sum(matL)))
  matQq <- apply(matQ, 2, function(x) normalise(x, colSums(matL), sum(matL)))

  #####

  # results in the list result

  result <- list()

  # calculate observed statistic for quantitative and categorical matrices
  obsc <- t(matR)%*%(matL%*%matQ)
  obsq <- t(matRq)%*%(matL%*%matQq)

  # if both variables are categorical (index==4), cell of obsc is taken;
  # if at least one of the variables is quantitative: take obsq
  obs <- matrix(sapply(1:prod(dim(index)), function(x)
      if (as.numeric(index)[x] == 4) as.numeric(obsc)[x] else
          as.numeric(obsq)[x]), nrow=nrow(index), ncol=ncol(index))

  # create as many clusters as number of available cores-1
  clus <- makeCluster(nrcor)

  # export needed data to clusters
  clusterExport(clus,list("matL","matR","matQ","matRq","matQq","index",
                          "obs","calstat"),
                envir=environment())

  # use parLapply to count how often empirical statistical values >= obs
  # in the case of row-wise permutation
  countr <- parLapply(clus, rep("r",npermut), function(x)
                        calstat(R=matR, Rq=matRq, L=matL, Q=matQ, Qq=matQq,
                        o=obs, permut=x, ind=index))
  countr <- Reduce(function(x,y){x+y}, countr)
  # in the case of column-wise permutation
  countc <- parLapply(clus, rep("c",npermut), function(x)
                        calstat(R=matR, Rq=matRq, L=matL, Q=matQ, Qq=matQq,
                        o=obs, permut=x, ind=index))
  countc <- Reduce(function(x,y){x+y}, countc)

  # close the clusters
  stopCluster(clus)

  # save the statistical value dependent on the index
  # if both variables are categorical (index==4), the N statistic amounts to
  # the already calculated statistic
  # if at least one variables is quantitative, the correlation coefficient is
  # computed by dividing the statistic by the sum over matrix L
  result$stat <- matrix(sapply(1:prod(dim(index)), function(x)
                            if (as.numeric(index)[x] == 4) {as.numeric(obs)[x]}
                            else {as.numeric(obs)[x]/sum(matL)}),
                        nrow=nrow(index), ncol=ncol(index),
                        dimnames=list(colnames(matR), colnames(matQ)))

  # calculate p-value of row-wise and column-wise permutation
  # independently from each other (equation see Vignette)
  pvaluer <- 2 * pmin((countr+1)/(npermut+1), (1-(countr+1)/(npermut+1)))
  pvaluec <- 2 * pmin((countc+1)/(npermut+1), (1-(countc+1)/(npermut+1)))

  # special case if variance of at least one variable equals zero:
  # all pvalues of this variable are 1
  pvaluer[which(rowVars(t(matR))==0),] <- 1
  pvaluer[,which(rowVars(t(matQ))==0)] <- 1
  pvaluec[which(rowVars(t(matR))==0),] <- 1
  pvaluec[,which(rowVars(t(matQ))==0)] <- 1

  # adjust p-values and choose maximal pvalue
  adj.pvaluer <- as.numeric(p.adjust(pvaluer, method=padjust))
  adj.pvaluec <- as.numeric(p.adjust(pvaluec, method=padjust))
  result$adj.pvalue <- matrix(pmax(adj.pvaluer, adj.pvaluec),
                              ncol=ncol(result$stat),
                              nrow=nrow(result$stat),
                              dimnames=list(rownames(result$stat),
                                            colnames(result$stat)))

  # investigate if the maximum p-value is taken from row- or column-wise
  # permutation and choose respective non-adjusted p-value
  is.r=(result$adj.pvalue == adj.pvaluer)
  pvalue <- rep(0, length(is.r))
  for (i in 1:length(is.r)) {
    if (is.r[i] == TRUE) {pvalue[i] <- pvaluer[i]}
    else {pvalue[i] <- pvaluec[i]}
  }
  
  # statistical tests
  stattest <- matrix('corr coeff', dimnames = list(colnames(matR), colnames(matQ)), 
                     nrow = length(indexR), ncol=length(indexQ))
  stattest[index==4] <- 'Chi-square related test'
  
  # assign p-values
  result$pvalue <- matrix(pvalue, ncol=ncol(result$stat),
                          nrow=nrow(result$stat),
                          dimnames=list(rownames(result$stat),
                                        colnames(result$stat)))

  # add call of the function to result
  result$call <- match.call()
  result$stattest <- stattest
  # add number of analysed genes to result
  if (exprvar != 1) {result$ngenes <- ngenes} else result$ngenes <- "all"
  # add number of permutations to result
  result$npermut <- npermut
  # add multiple hypothesis adjustment method to result
  result$adjust.method <- padjust
  class(result) <- "stat"
  return(result)
}

# simultaneous visualisation of stat and ord: vis

# input:
# [Stat] object of stat function;
# [Ord] object of ord function;
# [alpha] defines significance level of pvalues (default:0.05);
# [xaxis] and [yaxis] define axes of yination (default:1 and 2, resp.);
# [col] three cols: first col defines non-significant variables, second/third col
# define significant positive/negative asscoiations, respectively
# (default: "gray",transblue,transred);
# [alphatrans] defines degree of transparency of the second and third col;
# [cex] defines text size (default=1);
# [rangex, rangey] define ize of the plot (default=2);
# [...] additional features can be added

vis <- function(Stat, Ord=NULL, alpha=0.05, xaxis=1, yaxis=2,
                col=c("gray", transblue, transred),
                alphatrans=0.5, cex=1, rangex=2, rangey=2, ...) {

    if (missing(Stat)) {
        print("Only ordination will be plotted.")
    }
    else if (!inherits(Stat, "stat")) {
        stop("object of class stat is required here")
    }

    if (!inherits(Ord, "ord")) {
        stop("object of class ord is required here")
    }
    if (length(Ord$eig) < 2) stop("The fourthcorner matrix should be factorized
        into at least two eigenvectors.")

    # col: neutral, positive, negative; alphatrans: level of transparency
    transblue=rgb(0, 0, 1, alpha=alphatrans)
    transred=rgb(1, 0, 0, alpha=alphatrans)
    
    # size of text
    cex <- par("cex") * cex

    # save coordinates of rows and columns of Ord
    rowcoor <- Ord$l1[,c(xaxis,yaxis)]
    colcoor <- Ord$c1[,c(xaxis,yaxis)]

    # span the coordinate system
    plot(rangex * range(min(rowcoor[,1], colcoor[,1]), max(rowcoor[,1], colcoor[,1])),
         rangey * range(min(rowcoor[,2], colcoor[,2]), max(rowcoor[,2], colcoor[,2])),
         xlab=paste("axis", xaxis),
         ylab=paste("axis", yaxis),
         type='n',
         bty='l')

    # order rows of rowcoor and colcoor alphabetically
    rowcoor <- rowcoor[order(rownames(rowcoor)),]
    colcoor <- colcoor[order(rownames(colcoor)),]

    # determine positive and negative significant associations
    if (missing(Stat)) {
        text(rowcoor[,1], rowcoor[,2], row.names(rowcoor), cex=cex)
        text(colcoor[,1], colcoor[,2], row.names(colcoor), cex=cex)
    }
    else {
        possig <- which(Stat$adj.pvalue <= alpha & Stat$stat > 0, arr.ind=TRUE)
        negsig <- which(Stat$adj.pvalue <= alpha & Stat$stat < 0, arr.ind=TRUE)
        # save all significant associations
        sig <- list(unique(c(possig[,1], negsig[,1])),
                    unique(c(possig[,2], negsig[,2])))
        # if there are significant associations:
        # visualise segments between significant variables in cols[2,3]
        if (length(sig[[1]]) > 0) {
            if (nrow(possig) > 0) {
                segments(rowcoor[possig[,1],1], rowcoor[possig[,1],2],
                         colcoor[possig[,2],1], colcoor[possig[,2],2], lty=1, lwd=2,
                         col=col[2])
            }
            if (nrow(negsig) > 0) {
                segments(rowcoor[negsig[,1],1], rowcoor[negsig[,1],2],
                         colcoor[negsig[,2],1], colcoor[negsig[,2],2], lty=1, lwd=2,
                         col=col[3])
            }

            # visualize variables without significant associations in col[1]
            if (length(row.names(rowcoor)[-sig[[1]]]) > 0) {
                text(rowcoor[-sig[[1]],1], rowcoor[-sig[[1]],2],
                     row.names(rowcoor)[-sig[[1]]], cex=cex, col=col[1])
            }
            if (length(row.names(colcoor)[-sig[[2]]]) > 0) {
                text(colcoor[-sig[[2]],1], colcoor[-sig[[2]],2],
                     row.names(colcoor)[-sig[[2]]], cex=cex, col=col[1])
            }

            # visualize significant variables in black
            if (length(row.names(rowcoor)[sig[[1]]]) > 0) {
                text(rowcoor[sig[[1]],1], rowcoor[sig[[1]],2],
                     row.names(rowcoor)[sig[[1]]], cex=cex)
            }
            if (length(row.names(colcoor)[sig[[2]]]) > 0) {
                text(colcoor[sig[[2]],1], colcoor[sig[[2]],2],
                     row.names(colcoor)[sig[[2]]], cex=cex)
            }

            # if there are no significant associations, show all in col[1]
        } else {
            text(rowcoor[,1], rowcoor[,2], row.names(rowcoor),
                 cex=cex, col=col[1])
            text(colcoor[,1], colcoor[,2], row.names(colcoor),
                 cex=cex, col=col[1])
        }
    }
}

\name{covRNA-package}
\alias{covRNA}
\title{The covRNA package}
\description{
covRNA (covariate analysis of RNA-Seq data) is a fast and user-friendly R package which implements fourthcorner analysis and RLQ of transcriptomic data. 

Gene expression data normally comes with covariates of the samples and of the genes. To analyze associations between sample and gene covariates, the fourthcorner analysis tests the statistical significance of the associations by permutation tests while the RLQ visualizes associations within and be-tween the covariates.

The fourthcorner analysis and RLQ implemented in the ade4 package are adapted to easily analyze large-scale transcriptomic data. (1) Runtime and storage space are significantly reduced, (2) the analysis accounts for tran-scriptome-specific shapes of the empirical permutation distributions, (3) the analysis is rendered user-friendly by supplying automation, simple design-ing of plots and unsupervised gene filtering.

To cite covRNA, please use citation("covRNA"). For further details, please refer to the vignette by openVignette("covRNA") and the man pages.
}
\details{
\tabular{ll}{
Package: \tab covRNA\cr
Type: \tab Package\cr
License: \tab GPL (>=2)\cr
LazyLoad: \tab yes\cr
}
}
\author{
Lara Urban

Maintainer: Lara Urban <lara.h.urban@ebi.ac.uk>
}
\references{
To be announced soon.
}

\name{plot.ord}
\alias{plot.ord}
\title{Plot RLQ for transcriptomic data}
\description{
The function plot can visualise different features of an ord object by adjusting the argument "feature". By default, a barplot of the variance explained by the axes of the RLQ is plotted (see arguments).
}
\usage{
\method{plot}{ord}(x, feature="variance", xaxis=1, yaxis=2, cex=1, range=2, ...)
}
\arguments{
  \item{x}{An object of class ord that shall be visualised by ordination.}
  \item{feature}{Defines which features of the object shall be visualised: "columns L","rows L", "columns R" and "columns Q" visualise the respective variables as oridnation, "variance" shows a barplot of the variance explained by the axes, "correlation circle R" and "correlation circle Q" visualise the projection of the original space into the ordination space.}
  \item{xaxis, yaxis}{Define which axes of ordination shall be shown by x- and y-axis, respectively.}
  \item{cex}{Defines size of covariate text.}
  \item{range}{The range of the axes can be extended or reduced, e.g. for the case that not all covariates are visible in the default setting.}
  \item{...}{More plotting parameters can be added.}
}
\value{Plot of RLQ.}
\author{Lara Urban}
\examples{
ordBaca <- ord(Baca)
plot(ordBaca)
}

\name{stat}
\alias{stat}
\title{Fourthcorner analysis for transcriptomic data}
\description{
The fourthcorner analysis tests for significant associations between each sample covariate and each gene covariate by statistical permutation tests. The sample and gene covariates can be categorical and/or quantitative.

The input has to be given as dataframe or matrix. Dataframe/matrix L [n x p] contains transcriptomic data of p samples across n genes, dataframe/matrix R [n x m] contains m gene covariates across the n genes and dataframe/matrix Q [p x s] contains s sample covariates across the p samples. Alternatively, objects of the class ExpressionSet (with assayData, phenoData and featureData) can be used as input. If the argument ExprSet is missing, the function will use the dataframes/matrices R, L and Q as input.

The number of permutations is set to 9999 per default to assure significance of p-values after multiple testing correction. As computation time increases with size of the matrices/dataframes and with number of permutations, parallelization across multiple cores is highly recommended. Per default, all except one CPU cores on the current host are used.

Genes can be filtered with respect to their expression variance before analysis (argument exprvar); the function will automatically discard the gene covariates which do not annotate any of the remaining genes.

Warning: If R and Q are given as matrices, they will be converted to dataframes at the beginning of the function.

Warning: If R or Q is missing, it will be replaced by an identity matrix.
}
\usage{
stat(ExprSet, R=NULL, L=NULL, Q=NULL, npermut=9999, padjust="BH",
         nrcor=detectCores()-1, exprvar=1)
}

\arguments{
  \item{ExprSet}{An ExpressionSet of the \emph{Biobase} package.
  The ExpressionSet is used as default input. If no ExpressionSet is given, the individual dataframes/matrices R, L and Q can be used as input.}
  \item{R}{A dataframe/matrix containing information about each gene. The number of rows in R must match the number of rows in L. If R is missing, it will be replaced by an identity matrix [n x n].}
  \item{L}{A dataframe/matrix of gene expression values of genes across samples.}
  \item{Q}{A dataframe/matrix containing information about each sample. The number of rows in Q must match the number of columns in L. If Q is missing, it will be replaced by an identity matrix [p x p].}
  \item{npermut}{The number of permutations.}
  \item{padjust}{The method of multiple testing adjustment of the pvalues, see p.adjust.methods for all methods implemented in R.}
  \item{nrcor}{The number of cores to be used.}
  \item{exprvar}{The fraction of most variably expressed genes to take into account. If the functions `stat` and `ord` shall be combined, this value has to be the same in both analyses.}
}

\details{
Dependent on the covariate combination, a statistic is calculated based on matrix multiplication of the three tables. This statistic amounts to a correlation coefficient for the association between quantitative-quantitative and quantitative-categorical variables and to a Chi2-related statistic for the association between categorical-categorical variables.
}
\value{
The function returns a list of class stat where:
  \item{stat}{is a cross table (m x s) with the values of the original statistical tests per covariate combination.}
  \item{pvalue, adj.pvalue}{are cross tables (m x s) which contain the p-values and adjusted p-values, respectively, of the permutation tests per covariate combination.}
  \item{adjust.method}{shows the applied multiple testing adjustment method.}
  \item{npermut}{gives the number of permutations per permutation test.}
  \item{ngenes}{gives the number of analysed genes ("all" in the case of no filtering of the genes).}
  \item{call}{gives the original call of the function.}
}
\author{Lara Urban}
\examples{
data(Baca)
statBaca <- stat(ExprSet = Baca, npermut = 999, padjust = "BH", nrcor = 2, exprvar = 1)
statBaca$adj.pvalue
plot(statBaca)
}

\name{plot.stat}
\alias{plot.stat}
\title{Plot the fourthcorner analysis for transcriptomic data}
\description{
The function plot produces a cross table of the gene and sample covariates of a stat object. Colours indicate positive/negative significance or absence of significance of the assciations (per default: white for non-significant, red for negative significant and red for positive significant associations).
}

\usage{
\method{plot}{stat}(x, col=c("lightgrey","deepskyblue","red"), sig=TRUE,
                    alpha=0.05, show=c("adj","non-adj"), cex=1, 
                    ynames, xnames, ytext=1, xtext=1, shiftx=0, shifty=0, ...)
}

\arguments{
  \item{x}{An object of class stat that shall be visualised as a cross table.}
  \item{col}{A vector of three colours. The first colour represents non-significant, the second positive significant, the third negative significant associations in the cross table.}
  \item{sig}{If TRUE (default), only covariates involved in at least one significant association are plotted.}
  \item{alpha}{The significance level.}
  \item{show}{'adj' or 'non-adj' indicate if adjusted or raw p-values shall be plotted, respectively.}
  \item{cex}{The magnitude of the text in the cross table.}
  \item{ynames, xnames}{Row and column names of the cross table. By default, the column names of R and Q are used, respectively.}
  \item{ytext, xtext}{Rotation of the row and column names of the cross table.}
  \item{shifty, shiftx}{Shift of the row and column names to the right or to the left.}
  \item{...}{More plotting parameters can be added.}
}
\value{Plot of fourthcorner analysis.}
\author{Lara Urban}
\examples{
statBaca <- stat(Baca, nrcor = 2)
plot(statBaca)
}

\name{vis}
\alias{vis}
\title{Simultaneous visualisation of transcriptomic data by combining fourthcorner analysis and RLQ}
\description{
The vis function simultaneously visualizes the results of the functions stat and ord. Firstly, all covariates of R and Q are visualized by ordination in one plot; covariates involved in at least one significant association are shown in black, other covariates are shown in gray. Then, all covariates that are significantly associated according to stat are connected by lines which color represents the character of their significance.
}
\usage{
vis(Stat, Ord=NULL, alpha=0.05, xaxis=1, yaxis=2, col=c("gray", transblue, transred),
    alphatrans=0.5, cex=1, rangex=2, rangey=2, ...)
}
\arguments{
  \item{Stat}{An object of class stat.}
  \item{Ord}{An object of class ord. The objects stat and ord should have the same value ngenes.}
  \item{alpha}{The significance level.}
  \item{xaxis, yaxis}{Define which axes of ordination shall be shown by x- and y-axis, respectively.}
  \item{col}{A vector of three colors. The first color represents non-significant variables, the second positive significant, the third negative significant associations.}
  \item{alphatrans}{Defines degree of transparency of the second and third color.}
  \item{cex}{The magnitude of the text in the ordination.}
  \item{rangex, rangey}{The range of the x axis and y axis can be extended or reduced, e.g. for the case that not all covariates are visible in the default setting.}
  \item{...}{More plotting parameters can be added.}
}
\value{Plot of fourthcorner analysis and RLQ. }
\author{Lara Urban}
\examples{
data(Baca)
statBaca <- stat(Baca, nrcor = 2)
ordBaca <- ord(Baca)
vis(Stat = statBaca, Ord = ordBaca)
vis(Ord = ordBaca)
}

\name{Baca dataset}
\docType{data}
\alias{Baca}
\title{The Baca dataset}
\description{
The integrated Baca dataset contains the ExpressionSet Baca; its assayData contains deep sequenced RNA-Seq data of Bacillus anthracis under four stress conditions (with four replicates per stress conditions). The raw sequence reads derive from Passalacqua et al. (2012) and are availaible at Gene Expression Omnibus (GEO, accession number GSE36506). We have already mapped, counted and DESeq2 normalised these counts. The phenoData assigns the stress condition, i.e. ctrl, cold, salt and alcohol stress, to the samples. The featureData contains COG annotations of the genes.
}
\usage{Baca}
\value{ExpressionSet}
\format{ExpressionSet}
\examples{
data(Baca)
fData(Baca)
pData(Baca)
exprs(Baca)
}
\source{GEO GSE36506}
\references{
Passalacqua, K. D., Varadarajan, A., Weist, C., Ondov, B. D., Byrd, B. et al. (2012) \emph{Strand-Specific RNA-Seq Reveals Ordered Patterns of Sense and Antisense Transcription in Bacillus anthracis}. PLoS ONE, 7(8):e43350.
}
\keyword{dataset}

\name{ord}
\alias{ord}
\title{RLQ for transcriptomic data}
\description{
The RLQ visualises the association between and within sample and gene covariates by ordination. It applies generalized singular value decomposition (GSVD) to the fourthcorner matrix, which contains the associations between the sample and gene covariates. This is realised by eigendecomposition of the covariance matrices of the fourthcorner matrix. The name RLQ refers to the three dataframes R, L and Q to be analyzed. 
The function `ord` automates the `rlq` function of the `ade4` package.

The input has to be given as dataframe or matrix. Dataframe/matrix L [n x p] contains transcriptomic data of p samples across n genes, dataframe/matrix R [n x m] contains m gene covariates across the n genes and dataframe/matrix Q [p x s] contains s sample covariates across the p samples. Alternatively, objects of the class ExpressionSet (with assayData, phenoData and featureData) can be used as input. If the argument ExprSet is missing, the function will use the dataframes/matrices R, L and Q as input.

Genes can be filtered with respect to their expression variance before analysis (argument exprvar); the function will automatically discard the gene covariates which do not annotate any of the remaining genes.

Warning: If R and Q are given as matrices, they will be converted to dataframes at the beginning of the function.

Warning: If R or Q is missing, it will be replaced by an identity matrix. Then, a principal component analysis of this matrix will be performed what might be time-consuming, depending on the size of the identity matrix.
}
\usage{
ord(ExprSet, R=NULL, L=NULL, Q=NULL, exprvar=1, nf=2)
}
\arguments{
  \item{ExprSet}{An ExpressionSet of the \emph{Biobase} package.
The ExpressionSet is used as default input. If no ExpressionSet is given, the individual dataframes/matrices R, L and Q can be used as input.}
  \item{R}{A dataframe/matrix containing information about each gene. The number of rows in R must match the number of rows in L. If R is missing, it will be replaced by an identity matrix [n x n].}
  \item{L}{A dataframe/matrix of gene expression values of genes across samples.}
  \item{Q}{A dataframe/matrix containing information about each sample. The number of rows in Q must match the number of columns in L. If Q is missing, it will be replaced by an identity matrix [p x p].}
  \item{exprvar}{The fraction of most variably expressed genes to take into account. If the functions `stat` and `ord` shall be combined, this value has to be the same in both analyses.}
  \item{nf}{The number of axes to be considered by ordination.}
}
\details{
The function automates the following steps. Firstly, Correspondence Analysis is applied to gene expression table L. Either Principal Component Analysis (only quantitative variables), Multiple Correspondence Analysis (only categorical variables) or Hillsmith analysis (quantitative and categorical variables) are applied to the covariate tables R and Q. Secondly, RLQ is applied to the results of these ordination methods.
}
\value{
The function returns a list ob class ord where:
  \item{call}{gives the original call of the function.}
  \item{rank}{gives the rank.}
  \item{nf}{gives number of axes to be considered by ordination.}
  \item{RV}{gives the RV coefficient.}
  \item{eig}{gives a vector of the eigenvalues.}
  \item{variance}{gives the variance explained by the axes.}
  \item{lw}{gives the row weights of the fourthcorner table.}
  \item{cw}{gives the column weights of the fourthcorner table.}
  \item{lw}{gives the row weights of the fourthcorner table.}
  \item{tab}{gives the fourthcorner table.}
  \item{li}{gives the coordinates of the covariates of R.}
  \item{l1}{gives the normed scores of the covariates of R.}
  \item{co}{gives the coordinates of the covariates of Q.}
  \item{c1}{gives the normed scores of the covariates of Q.}
  \item{lR}{gives the row coordinates of R.}
  \item{mR}{gives the normed row scores of R.}
  \item{lQ}{gives the row coordinates of Q.}
  \item{mQ}{gives the normed row scores of Q.}
  \item{aR}{gives projection of axis onto co-inertia axis of R.}
  \item{aR}{gives projection of axis onto co-inertia axis of Q.}
  \item{ngenes}{gives the number of analysed genes.}
}
\author{Lara Urban}
\examples{
data(Baca)
ordBaca <- ord(ExprSet = Baca, exprvar = 1, nf = 2)
ls(ordBaca)
plot(ordBaca)
}
